@@ -1,9 +1,9 @@
 package Data::Sah::Compiler::Prog::TH;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-02-10'; # DATE
+our $DATE = '2020-02-11'; # DATE
 our $DIST = 'Data-Sah'; # DIST
-our $VERSION = '0.905'; # VERSION
+our $VERSION = '0.906'; # VERSION
 
 use 5.010;
 use strict;
@@ -89,7 +89,7 @@ sub set_tmp_data_term {
         $cd->{data_term} = $t;
     }
     local $cd->{_debug_ccl_note} = 'set temporary data term';
-    $c->add_ccl($cd, "(".$c->expr_assign($t, $expr). ", ".$c->true.")",
+    $c->add_ccl($cd, "(".$c->expr_set($t, $expr). ", ".$c->true.")",
                 {err_msg => ''});
 }
 
@@ -194,7 +194,7 @@ Data::Sah::Compiler::Prog::TH - Base class for programming-language emiting comp
 
 =head1 VERSION
 
-This document describes version 0.905 of Data::Sah::Compiler::Prog::TH (from Perl distribution Data-Sah), released on 2020-02-10.
+This document describes version 0.906 of Data::Sah::Compiler::Prog::TH (from Perl distribution Data-Sah), released on 2020-02-11.
 
 =for Pod::Coverage ^(compiler|clause_.+|handle_.+|gen_.+|set_tmp_data_term|restore_data_term)$
 
