@@ -1,7 +1,9 @@
 package Data::Sah::Type::BaseType;
 
-our $DATE = '2020-02-10'; # DATE
-our $VERSION = '0.905'; # VERSION
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2020-02-11'; # DATE
+our $DIST = 'Data-Sah'; # DIST
+our $VERSION = '0.906'; # VERSION
 
 # why name it BaseType instead of Base? because I'm sick of having 5 files named
 # Base.pm in my editor (there would be Type::Base and the various
@@ -16,6 +18,11 @@ use Role::Tiny;
 #use Sah::Schema::Common;
 #use Sah::Schema::Sah;
 
+our $sch_filter_elem = ['any', {of=>[
+    ['str', {req=>1}, {}],
+    ['array', {req=>1, len=>2, elems=>[ ['str',{req=>1},{}], ['hash',{req=>1}, {}] ]}, {}],
+]}, {}];
+
 requires 'handle_type';
 
 has_clause 'v',
@@ -67,15 +74,15 @@ has_clause 'default',
     },
     ;
 has_clause 'prefilters',
-     v => 2,
-     tags       => ['filter'],
-     prio       => 10,
-     schema     => ['array' => {of=>['str', {req=>1}, {}]}, {}],
-     attrs      => {
-         temp => {
-         },
-     }
-     ;
+    v => 2,
+    tags       => ['filter'],
+    prio       => 10,
+    schema      => ['array' => {of=>$sch_filter_elem}, {}],
+    attrs      => {
+        temp => {
+        },
+    }
+    ;
 has_clause 'default_lang',
     v => 2,
     tags       => ['meta', 'defhash'],
@@ -157,10 +164,8 @@ has_clause 'postfilters',
     v => 2,
     tags       => ['filter'],
     prio       => 90,
-    schema     => ['array' => {req=>1, of=>['str', {req=>1}, {}]}, {}],
+    schema     => ['array' => {req=>1, of=>$sch_filter_elem}, {}],
     attrs      => {
-        temp => {
-        },
     }
     ;
 has_clause 'examples',
@@ -191,7 +196,7 @@ Data::Sah::Type::BaseType - Base type
 
 =head1 VERSION
 
-This document describes version 0.905 of Data::Sah::Type::BaseType (from Perl distribution Data-Sah), released on 2020-02-10.
+This document describes version 0.906 of Data::Sah::Type::BaseType (from Perl distribution Data-Sah), released on 2020-02-11.
 
 =for Pod::Coverage ^(clause_.+|clausemeta_.+)$
 
